#!/bin/sh

# mkipk.sh
# generates an ipkg for BilliardZ

# Based off the mkipk.sh by Bill Kendrick <bill@newbreedsoftware.com>

# Revised by Christian Hammond <chipx86@gnupdate.org>

. ./mkstuff.conf

DIST=$TMPDIR/BUILD/DIST
CONTROL=$DIST/CONTROL/control

echo "SETTING UP"
mkdir -p $TMPDIR/BUILD
mkdir -p $TMPDIR/BUILD/DIST/CONTROL

echo
echo "COPYING SOURCES INTO BUILD AREA"
cp -av src images billiardz.arm.pro $TMPDIR/BUILD/

# Remove the xcf files
$RM $TMPDIR/BUILD/images/*.xcf
$RM -rf $TMPDIR/BUILD/images/.xvpics
$RM -rf $TMPDIR/BUILD/images/CVS
$RM -rf $TMPDIR/BUILD/src/CVS
$RM -rf $TMPDIR/BUILD/CVS
cd $TMPDIR/BUILD

echo
echo "MAKING SURE BINARY EXISTS"
. /home/chipx86/bin/dev-arm-qpe.sh
tmake billiardz.arm.pro -o Makefile
make clean
make

if [ $? -ne 0 ]; then
	echo
	echo "ERROR IN COMPILE"
	exit 1
fi

cd ../..

echo 
echo "CREATING CONTROL FILE"

echo "Package: $PACKAGE" > $CONTROL
echo "Priority: optional" >> $CONTROL
echo "Version: $VER" >> $CONTROL
echo "Section: games" >> $CONTROL
echo "Architecture: $ARCH" >> $CONTROL
echo "Maintainer: Christian Hammond <chipx86@gnupdate.org>" >> $CONTROL
echo "Description: Billiards for the Zaurus SL-5000D and SL-5500" >> $CONTROL

echo
echo "COPYING DATA FILES"

mkdir -p $DIST/opt/QtPalmtop/share/billiardz
mkdir $DIST/opt/QtPalmtop/share/billiardz/images/
cp -R $TMPDIR/BUILD/images/* $DIST/opt/QtPalmtop/share/billiardz/images/

echo
echo "CREATING BINARIES"

mkdir -p $DIST/opt/QtPalmtop/bin/
cp $TMPDIR/BUILD/billiardz $DIST/opt/QtPalmtop/bin/

echo
echo "STRIPPING THE BINARY"
arm-linux-strip $DIST/opt/QtPalmtop/bin/billiardz

echo
echo "CREATING ICON AND DESKTOP FILE"

mkdir -p $DIST/opt/QtPalmtop/pics/
cp billiardz.png $DIST/opt/QtPalmtop/pics/billiardz.png

mkdir -p $DIST/opt/QtPalmtop/apps/Games/
DESKTOP=$DIST/opt/QtPalmtop/apps/Games/billiardz.desktop
echo "[Desktop Entry]" > $DESKTOP
echo "Comment=Billiards for Zaurus" >> $DESKTOP
echo "Exec=billiardz" >> $DESKTOP
echo "Icon=billiardz" >> $DESKTOP
echo "Type=Application" >> $DESKTOP
echo "Name=BilliardZ" >> $DESKTOP


echo
echo "CREATING IPK..."

ipkg-build $DIST

echo
echo "CLEANING UP"

$RM -r $TMPDIR

echo

