#include "Ball.h"
#include <math.h>
#include <string.h>
#include <stdlib.h>

Ball::Ball(SDL_Surface *screen, PoolTable *table, int num)
	: Sprite(screen, IMAGE_FIRST_BALL + num, true),
	  poolTable(table), player(0), number(num)
{
	if (num == 0)
		type = BALLTYPE_CUEBALL;
	else if (num == 8)
		type = BALLTYPE_8BALL;
	else if (num >= 1 && num <= 7)
		type = BALLTYPE_SOLID;
	else if (num >= 9 && num <= 15)
		type = BALLTYPE_STRIPED;
	else
		type = BALLTYPE_UNSET;
}

Ball::~Ball()
{
}

const char *
Ball::getName() const
{
	return "Ball";
}

BallType
Ball::getType() const
{
	return type;
}

int
Ball::getNumber() const
{
	return number;
}

void
Ball::setPlayer(int player)
{
	this->player = player;
}

int
Ball::getPlayer() const
{
	return this->player;
}

void
Ball::setVector(Fixed x, Fixed y)
{
	vec.setVec(x, y);

	if (x != 0 || y != 0)
		setAnimating(true);
	else
		setAnimating(false);
}

void
Ball::setVector(const Vec2D &vec)
{
	setVector(vec.getDx(), vec.getDy());
}

const Vec2D &
Ball::getVector() const
{
	return vec;
}

Fixed
Ball::getVectorX() const
{
	return vec.getDx();
}

Fixed
Ball::getVectorY() const
{
	return vec.getDy();
}

Fixed
Ball::getRadius() const
{
	return 6;
}

Fixed
Ball::getDiameter() const
{
	return Fixed(2 * M_PI) * getRadius();
}

Fixed
Ball::getCenterX() const
{
	return getX() + getRadius();
}

Fixed
Ball::getCenterY() const
{
	return getY() + getRadius();
}

Fixed
Ball::getMomentum() const
{
	return 0.17;
}

bool
Ball::isStopped() const
{
	return !isAnimating();
}

bool
Ball::isMoving() const
{
	return isAnimating();
}

Fixed
Ball::collisionPrediction(Ball *ball)
{
	Fixed deltaVx, deltaVy, dx, dy;
	Fixed a, b, c, d;
	double da, db, dc, dd;
	double result;

	/* Initial (pre-collision) motion vector */
	deltaVx = getVectorX() - ball->getVectorX();
	deltaVy = getVectorY() - ball->getVectorY();

	/* A temporary vector */
	dx = getX() - ball->getX();
	dy = getY() - ball->getY();

	/* Main body of evaluation of collision time */
	a = deltaVx.square() + deltaVy.square();

	if (a <= 0)
		return -1;

	b = (dx * deltaVx) + (dy * deltaVy);
	c = dx.square() + dy.square() - (getRadius() * 2).square();
	d = b.square() - a * c;

	if (d <= 0)
		return -1;

	da = a.doubleVal();
	db = b.doubleVal();
	dc = c.doubleVal();
	dd = (db * db) - da * dc; //d.doubleVal();
	
	result = (-db - sqrt(dd)) / da;

	return result;
}

bool
Ball::collidesWith(Sprite *sprite)
{
	Ball *ball;
	Fixed dx, dy;
	Fixed distance;

	if (this == sprite)
		return false;

	if (strcmp(sprite->getName(), "Ball"))
		return Sprite::collidesWith(sprite);

	ball = (Ball *)sprite;

	dx = getCenterX() - ball->getCenterX();
	dy = getCenterY() - ball->getCenterY();

	distance = dx.hypot(dy);

	if (distance <= getRadius())
		return true;

	return false;
}

void
Ball::nextFrame()
{
}
