#ifndef _BALL_H_
#define _BALL_H_

typedef enum
{
	BALLTYPE_UNSET = 0,
	BALLTYPE_SOLID,
	BALLTYPE_STRIPED,
	BALLTYPE_CUEBALL,
	BALLTYPE_8BALL,
	BALLTYPE_ANY

} BallType;

#include "BilliardZ.h"

class PoolTable;
class Side;
class Sprite;

class Ball : public Sprite
{
	public:
		static const double deceleration = 0.028;
		static const double hitSideDecel = 0.020;
		static const double hitBallDecel = 0.010;

		Ball(SDL_Surface *screen, PoolTable *table, int num);
		~Ball();

		virtual const char *getName() const;

		BallType getType() const;

		int getNumber() const;

		void setPlayer(int player);
		int getPlayer() const;

		void setVector(Fixed x, Fixed y);
		void setVector(const Vec2D &vec);
		const Vec2D &getVector() const;
		Fixed getVectorX() const;
		Fixed getVectorY() const;

		Fixed getRadius() const;
		Fixed getDiameter() const;
		Fixed getCenterX() const;
		Fixed getCenterY() const;

		Fixed getMomentum() const;

		bool isStopped() const;
		bool isMoving() const;

		Fixed collisionPrediction(Ball *ball);

		virtual bool collidesWith(Sprite *sprite);

		virtual void nextFrame();

		void decel(Fixed f)
		{
			if (f >= getVector().abs())
				setVector(0, 0);
			else
				setVector(getVectorX() - (getVectorX() * f),
						  getVectorY() - (getVectorY() * f));
		}

	private:
		PoolTable *poolTable;

		BallType type;

		int player;
		int number;

		Vec2D vec;
};

#endif /* _BALL_H_ */
