#include "EightBallRules.h"

extern ImageLoader *imageLoader;

EightBallRules::EightBallRules(PoolGame *game)
	: Rules(game), firstBallHit(false)
{
}

void
EightBallRules::rackBalls()
{
	static const int BALL_ORDER[] = {
		8, 9, 0, 1, 7, 10, 12, 3, 11, 2, 6, 5, 14, 4, 13
	};

	Fixed y;
	int row, i;
	Fixed ballWidth, ballHeight;
	Fixed spacing;
	Ball *ball;
	PoolTable *table = getGame()->getTable();
	QList<Ball> inPlayBalls = table->getInPlayBalls();
	QList<Ball> tempBalls;

	ballWidth  = table->getCueBall()->getWidth();
	ballHeight = table->getCueBall()->getHeight();

	y = table->getY() + 105;

	spacing = 0;

	/* Let's re-order these so we can put them in the layout we want. */
	for (i = 0; i < 15; i++)
		tempBalls.append(inPlayBalls.at(BALL_ORDER[i]));

	for (row = 1, ball = tempBalls.first(); row <= 5; row++, y -= ballHeight)
	{
		for (i = 0; i < row; i++)
		{
			ball->setPlayer(-1);

			ball->moveTo(table->getX() + (table->getWidth() -
										  ((ballWidth + spacing) * row)) / 2 +
						 ((ballWidth + spacing) * i), y);

			ball = tempBalls.next();
		}
	}
}

BallResult
EightBallRules::ballInHole(Ball *ball, Hole *)
{
	Player *player = getGame()->getActivePlayer();

	if (ball->getType() == BALLTYPE_8BALL)
	{
		if (player->getNumBallsIn() < 7)
		{
			/* Got the 8 ball in before it's ready. */
			
			return PLAYER_LOST;
		}

		return PLAYER_WON;
	}
	else if (ball->getType() == BALLTYPE_CUEBALL)
	{
		return SCRATCHED;
	}
	else if (!firstBallHit)
	{
		SDL_Surface *stripePic, *solidPic;

		stripePic = imageLoader->load(IMAGE_FIRST_BALL + 15, true);
		solidPic  = imageLoader->load(IMAGE_FIRST_BALL + 7,  true);

		player->setBallType(ball->getType());
		player->setNextBallPic(ball->getType() == BALLTYPE_SOLID ?
							   solidPic : stripePic);

		if (!getGame()->isSinglePlayer())
		{
			getGame()->getInactivePlayer()->setBallType(
				(ball->getType() == BALLTYPE_SOLID ? BALLTYPE_STRIPED :
				 BALLTYPE_SOLID));

			getGame()->getInactivePlayer()->setNextBallPic(
				(ball->getType() == BALLTYPE_SOLID ? stripePic : solidPic));
		}

		firstBallHit = true;

		return VALID_BALL;
	}
	else if (player->getBallType() == ball->getType())
	{
		return VALID_BALL;
	}

	return INVALID_BALL;
}

BallResult
EightBallRules::ballHit(Ball *ball)
{
	if (!firstBallHit)
		return VALID_BALL;

	if (ball->getType() == getGame()->getActivePlayer()->getBallType())
		return VALID_BALL;

	return INVALID_BALL;
}
