/*
 * libmad - MPEG audio decoder library
 * Copyright (C) 2000-2001 Robert Leslie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Fixed.cpp,v 1.6 2002/09/22 23:44:05 chipx86 Exp $
 */

#include <math.h>
#include <bits/nan.h> /* Bad! */
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "Fixed.h"

#if 0
Fixed::Fixed(fixed_t num)
{
	set(num);
}
#endif

Fixed::Fixed(double num)
{
	if (isnan(num))
		set(F_NAN);
	else
		set(F_FIXED(num));
}

Fixed::Fixed(int num)
{
	set(F_FROMINT(num));
}

Fixed::Fixed(unsigned int num)
{
	set(F_FROMINT(num));
}

Fixed::~Fixed()
{
}

Fixed
Fixed::operator=(const Fixed &f)
{
	return set(f.value());
}

bool
Fixed::operator==(const Fixed &f) const
{
	return value() == f.value();
}

bool
Fixed::operator!=(const Fixed &f) const
{
	return value() != f.value();
}

bool
Fixed::operator<(const Fixed &f) const
{
	return value() < f.value();
}

bool
Fixed::operator<=(const Fixed &f) const
{
	return value() <= f.value();
}

bool
Fixed::operator>(const Fixed &f) const
{
	return value() > f.value();
}

bool
Fixed::operator>=(const Fixed &f) const
{
	return value() >= f.value();
}

Fixed
Fixed::operator+(const Fixed &f) const
{
	if (isNaN() || f.isNaN())
		return F_NAN;

	return Fixed().set(F_ADD(value(), f.value()));
}

Fixed
Fixed::operator-(const Fixed &f) const
{
	if (isNaN() || f.isNaN())
		return F_NAN;

	return Fixed().set(F_SUB(value(), f.value()));
}

Fixed
Fixed::operator*(const Fixed &f) const
{
	if (isNaN() || f.isNaN())
		return F_NAN;

	return Fixed().set(F_MUL(value(), f.value()));
}

Fixed
Fixed::operator/(const Fixed &f) const
{
	if (isNaN() || f.isNaN())
		return F_NAN;

	return Fixed().set((fixed_t)F_DIV(value(), f.doubleVal()));
}

void
Fixed::operator+=(const Fixed &f)
{
	if (isNaN() || f.isNaN())
		return;

	val = F_ADD(value(), f.value());
}

void
Fixed::operator-=(const Fixed &f)
{
	if (isNaN() || f.isNaN())
		return;

	val = F_SUB(value(), f.value());
}

void
Fixed::operator*=(const Fixed &f)
{
	if (isNaN() || f.isNaN())
		return;

	val = F_MUL(value(), f.value());
}

void
Fixed::operator/=(const Fixed &f)
{
	if (isNaN() || f.isNaN())
		return;

	val = (fixed_t)F_DIV(value(), f.doubleVal());
}

Fixed
Fixed::set(fixed_t fixed)
{
	if (fixed == F_NAN)
		val = F_NAN;
	else if (fixed < F_MIN)
	{
		printf("WARNING: %f < F_MIN\n", (double)fixed);
		val = F_MIN;
//		abort();
	}
	else if (fixed > F_MAX)
	{
		printf("WARNING: %f > F_MAX\n", (double)fixed);
		val = F_MAX;
//		abort();
	}
	else
		val = fixed;

	return *this;
}

double
Fixed::doubleVal() const
{
	if (isNaN())
		return NAN;

	return F_DOUBLE(val);
}

unsigned int
Fixed::intVal() const
{
	return F_INT(val);
}

int
Fixed::sintVal() const
{
	return F_INT(val);
}

fixed_t
Fixed::value() const
{
	return val;
}

bool
Fixed::isNaN() const
{
	return (val == F_NAN);
}

Fixed
Fixed::abs() const
{
	if (isNaN())
		return F_NAN;

	return Fixed().set(val < 0 ? -val : val);
}

Fixed
Fixed::sqrt() const
{
	if (isNaN())
		return F_NAN;

	if (val > 0)
	{
		double result = ::sqrt(doubleVal());

		if (errno == EDOM)
			return F_NAN;
		
		return result;
	}

	if (val < 0)
	{
		errno = EDOM;
		return F_NAN;
	}

	return 0;
}

Fixed
Fixed::hypot(const Fixed &f) const
{
	if (isNaN() || f.isNaN())
		return F_NAN;

	return ::hypot(doubleVal(), f.doubleVal());
}

Fixed
Fixed::cos() const
{
	if (isNaN())
		return F_NAN;

	return ::cos(doubleVal());
}

Fixed
Fixed::sin() const
{
	if (isNaN())
		return F_NAN;

	return ::sin(doubleVal());
}

Fixed
Fixed::atan() const
{
	if (isNaN())
		return F_NAN;

	return ::atan(doubleVal());
}

Fixed
Fixed::square() const
{
	if (isNaN())
		return F_NAN;

	return *this * *this;
}

Fixed
Fixed::neg() const
{
	if (isNaN())
		return F_NAN;

	return Fixed().set(-val);
}
