#include "ImageLoader.h"
#include <stdlib.h>

static const char *imageFiles[] =
{
	DATA_PREFIX "images/table.png",
	DATA_PREFIX "images/unsetball.png",
	DATA_PREFIX "images/cueball.png",
	DATA_PREFIX "images/1ball.png",
	DATA_PREFIX "images/2ball.png",
	DATA_PREFIX "images/3ball.png",
	DATA_PREFIX "images/4ball.png",
	DATA_PREFIX "images/5ball.png",
	DATA_PREFIX "images/6ball.png",
	DATA_PREFIX "images/7ball.png",
	DATA_PREFIX "images/8ball.png",
	DATA_PREFIX "images/9ball.png",
	DATA_PREFIX "images/10ball.png",
	DATA_PREFIX "images/11ball.png",
	DATA_PREFIX "images/12ball.png",
	DATA_PREFIX "images/13ball.png",
	DATA_PREFIX "images/14ball.png",
	DATA_PREFIX "images/15ball.png",
	DATA_PREFIX "images/ballshadow.png",
	DATA_PREFIX "images/top_side.png",
	DATA_PREFIX "images/topright_side.png",
	DATA_PREFIX "images/bottomright_side.png",
	DATA_PREFIX "images/bottom_side.png",
	DATA_PREFIX "images/bottomleft_side.png",
	DATA_PREFIX "images/topleft_side.png",
	DATA_PREFIX "images/hole.png",
	DATA_PREFIX "images/splashscreen.png",
	DATA_PREFIX "images/floor.png",
	DATA_PREFIX "images/tableshadow.png",
	DATA_PREFIX "images/curplayer_arrow.png",
	DATA_PREFIX "images/meter_bg.png",
	DATA_PREFIX "images/meter_bar.png"
};

/* Total number of images */
#define NUM_IMAGES (sizeof(imageFiles) / sizeof(*imageFiles))

ImageLoader::ImageLoader()
{
	images = new SDL_Surface *[NUM_IMAGES];

	for (Uint16 i = 0; i < NUM_IMAGES; i++)
		images[i] = NULL;
}

ImageLoader::~ImageLoader()
{
	for (Uint16 i = 0; i < NUM_IMAGES; i++)
	{
		if (images[i] != NULL)
			SDL_FreeSurface(images[i]);
	}

	delete [] images;

	images = NULL;
}

SDL_Surface *
ImageLoader::load(Uint16 num, bool alphaChannel = false)
{
	if (num >= NUM_IMAGES)
		return NULL;

	if (images[num] == NULL)
	{
		SDL_Surface *tempImage;
		SDL_Surface *image;

		tempImage = IMG_Load(imageFiles[num]);

		if (tempImage == NULL)
		{
			fprintf(stderr, "Unable to load graphics file `%s': %s\n",
					imageFiles[num], SDL_GetError());
			exit(1);
		}

		if (alphaChannel)
			image = SDL_DisplayFormatAlpha(tempImage);
		else
			image = SDL_DisplayFormat(tempImage);

		if (image == NULL)
		{
			fprintf(stderr,
					"Unable to convert file `%s' to display format: %s\n",
					imageFiles[num], SDL_GetError());
			exit(1);
		}

		SDL_FreeSurface(tempImage);

		images[num] = image;
	}

	return images[num];
}
