#include "Physics.h"

/* Kinetic Energy */
#define KE(m, vel) ((m * (vel * vel)) / 2)
#define momentum(m, v) (m * v)

Physics::Physics()
{
}

Physics::~Physics()
{
}

void
Physics::shootBall(Ball *ball, Fixed velocity, Fixed rad)
{
	Fixed vx, vy;

	vx = (velocity * 15).neg() * rad.sin();
	vy = (velocity * 15)       * rad.cos();

	ball->setVector(vx, vy);
}

void
Physics::ballToBallCollision(Ball *ball1, Ball *ball2, Ball *)
{
	Fixed totalDeceleration = Ball::deceleration;
	Fixed velDiffX, velDiffY;
	Fixed diffX, diffY;
	Fixed unitDx, unitDy;
	Fixed velX, velY;
	Fixed cos, sin;
	Fixed projection;
	Fixed speed;
	Fixed d;

	velX = ball1->getVectorX();
	velY = ball1->getVectorY();

	velDiffX = ball2->getVectorX() - velX;
	velDiffY = ball2->getVectorY() - velY;

	diffX = ball2->getX() - ball1->getX();
	diffY = ball2->getY() - ball1->getY();

	d = diffX.hypot(diffY);

	unitDx = diffX / d;
	unitDy = diffY / d;

	projection = velDiffX * unitDx + velDiffY * unitDy;

	velX += projection * unitDx;
	velY += projection * unitDy;

	ball1->setVector(velX, velY);

	ball2->setVector(ball2->getVectorX() - projection * unitDx,
					 ball2->getVectorY() - projection * unitDy);

	speed = velX.hypot(velY);

	cos = velX / speed;
	sin = velY / speed;

	totalDeceleration += Ball::hitBallDecel;

	if (speed < totalDeceleration)
	{
		velX  = 0;
		velY  = 0;
		speed = 0;
	}
	else
	{
		speed -= totalDeceleration;
		velX = speed * cos;
		velY = speed * sin;
	}

	ball1->setVector(velX, velY);
}

void
Physics::ballToSideCollision(Ball *ball, Side *side)
{
	Fixed velX, velY;
	Fixed hitX, hitY;
	Fixed modHit, projection;

	velX = ball->getVectorX();
	velY = ball->getVectorY();

	hitX = side->getDirX();
	hitY = side->getDirY();

#if 0
	printf("ballToSideCollision\n");

	if (hitX != 0)
		velX = ball->getVectorX().neg();

	if (hitY != 0)
		velY = ball->getVectorY().neg();
#endif

//	ball->setVector(velX, velY);

	if (hitX != 0 || hitY != 0)
	{
		modHit = hitX.hypot(hitY);
		projection = (hitX / modHit) * velX + (hitY / modHit) * velY;

		if (projection < 0)
		{
			velX -= projection * hitX * 2;
			velY -= projection * hitY * 2;

			ball->setVector(velX, velY);
		}
	}
}

void
Physics::updateBallPos(Ball *ball, bool sideCollision)
{
	Fixed totalDeceleration = Ball::deceleration;
	Fixed velX, velY;
	Fixed speed;
	Fixed sin, cos;

	velX = ball->getVectorX();
	velY = ball->getVectorY();
	
	speed = velX.hypot(velY);

	cos = velX / speed;
	sin = velY / speed;

	if (sideCollision)
		totalDeceleration += Ball::hitSideDecel;

	if (speed < totalDeceleration)
	{
		velX  = 0;
		velY  = 0;
		speed = 0;
	}
	else
	{
		speed -= totalDeceleration;
		velX   = speed * cos;
		velY   = speed * sin;
	}

	ball->setVector(velX, velY);
}
