#include "PlayerDisplay.h"

extern ImageLoader *imageLoader;

#define DISPLAY_WIDTH   20
#define BALL_OFFSET_X   (WIDTH - DISPLAY_WIDTH)
#define ARROW_OFFSET_X  (BALL_OFFSET_X - arrow->w - 2)
#define OFFSET_Y        5
#define SPACING_Y       15

PlayerDisplay::PlayerDisplay(SDL_Surface *_screen, PoolGame *_game)
	: screen(_screen), game(_game)
{
}

const char *
PlayerDisplay::getName() const
{
	return "PlayerDisplay";
}

void
PlayerDisplay::render(bool update)
{
	int playerCount = game->getPlayerCount();
	Player *player;
	SDL_Rect boundryRect, rect, shadowRect, arrowRect;
	SDL_Surface *ballShadow;
	SDL_Surface *arrow;

	/* Load the images */
	ballShadow = imageLoader->load(IMAGE_BALL_SHADOW,     true);
	arrow      = imageLoader->load(IMAGE_CURPLAYER_ARROW, true);

	/* Redraw the background. */
	boundryRect.x = ARROW_OFFSET_X;
	boundryRect.y = OFFSET_Y;
	boundryRect.w = DISPLAY_WIDTH + arrow->w + 2;
	boundryRect.h = (playerCount + 1) * SPACING_Y;

	game->getRoom()->render(boundryRect, false);

	/* Render the balls */
	rect.x = BALL_OFFSET_X;
	rect.w = DISPLAY_WIDTH;
	rect.h = SPACING_Y;

	shadowRect.x = rect.x;
	shadowRect.w = ballShadow->w;
	shadowRect.h = ballShadow->h;

	for (int i = 0; i < playerCount; i++)
	{
		player = game->getPlayer(i);

		rect.y       = OFFSET_Y + (SPACING_Y * i);
		shadowRect.y = rect.y;

		SDL_BlitSurface(ballShadow, NULL, screen, &shadowRect);
		SDL_BlitSurface(player->getNextBallPic(), NULL, screen, &rect);
	}

	/* Draw the current player arrow. */
	player = game->getActivePlayer();

	arrowRect.x = ARROW_OFFSET_X;
	arrowRect.w = arrow->w;
	arrowRect.h = arrow->h;
	arrowRect.y = OFFSET_Y + (SPACING_Y * player->getNumber());

	SDL_BlitSurface(arrow, NULL, screen, &arrowRect);

	if (update)
	{
		SDL_UpdateRect(screen, boundryRect.x, boundryRect.y, boundryRect.w,
					   boundryRect.h);
	}
}
