#include "PoolRoom.h"

#include <stdlib.h>

extern ImageLoader *imageLoader;

PoolRoom::PoolRoom(SDL_Surface *_screen, PoolGame *_game)
	: Sprite(_screen), game(_game)
{
	/* Setup the pool table. */
	table = new PoolTable(getScreen(), game);

	table->moveTo((F_WIDTH  - table->getWidth())  / 2,
				  (F_HEIGHT - table->getHeight()) / 2);

	/* Render this image. */
	prerender();

	/* Create the cue stick. */
	cueStick = new CueStick(getScreen(), game);
	cueStick->setTargetBall(table->getCueBall());
	cueStick->setAngle(178.0);
	cueStick->registerListener(table);
	cueStick->setVisible();

	/* Create the player display */
	playerDisp = new PlayerDisplay(getScreen(), game);

	/* Create the power meter */
	meter = new PowerMeter(getScreen(), game);
	meter->moveTo(215, 203);
}

PoolRoom::~PoolRoom()
{
	delete meter;
	delete playerDisp;
	delete cueStick;
	delete table;
}

const char *
PoolRoom::getName() const
{
	return "PoolRoom";
}

void PoolRoom::render(bool update)
{
	Sprite::render(false);

	table->render(false);
	cueStick->render(false);

	playerDisp->render(false);

	meter->render(false);

	if (update)
		SDL_Flip(getScreen());
}

PoolTable *
PoolRoom::getTable() const
{
	return table;
}

CueStick *
PoolRoom::getCueStick() const
{
	return cueStick;
}

PlayerDisplay *
PoolRoom::getPlayerDisplay() const
{
	return playerDisp;
}

PowerMeter *
PoolRoom::getPowerMeter() const
{
	return meter;
}

void
PoolRoom::prerender()
{
	SDL_Rect rect, shadowRect;
	SDL_Surface *image;
	SDL_Surface *tableShadow;

	rect = getTable()->getRect();

	/* Fill the image. */
	image = imageLoader->load(IMAGE_FLOOR, false);

	/* Add the table shadow. */
	tableShadow = imageLoader->load(IMAGE_TABLESHADOW, true);
	shadowRect.x = rect.x - 8;
	shadowRect.y = rect.y - 7;
	shadowRect.w = tableShadow->w;
	shadowRect.h = tableShadow->h;

	SDL_BlitSurface(tableShadow, NULL, image, &shadowRect);

	/* Add the table image. */
	SDL_BlitSurface(getTable()->getImage(), NULL, image, &rect);

	setImage(image);
}
