#ifndef _POOLTABLE_H_
#define _POOLTABLE_H_

#include "BilliardZ.h"
#include <qlist.h>

class Ball;
class CueStick;
class CueStickListener;
class Hole;
class PoolGame;
class Sprite;
class Side;

typedef enum
{
	TABLE_GREEN = 0,
	TABLE_RED

} TableTheme;

class PoolTable : public Sprite, public CueStickListener
{
	friend class PoolRoom;

	public:
		PoolTable(SDL_Surface *screen, PoolGame *game);
		~PoolTable();

		virtual const char *getName() const;

		void setTheme(TableTheme theme);
		TableTheme getTheme() const;

		void resetCueBall();

		void render(bool update = true);
		void render(SDL_Rect rect, bool update = true)
		{ Sprite::render(rect, update); }

		void renderShadows();
		void renderBalls(bool update = true);

		const QList<Ball> &getInPlayBalls() const;
		const QList<Sprite> &getObstacles() const;

		Ball *getCueBall() const { return cueBall; }

		PoolGame *getGame() const { return game; }

	protected:
		void moved();

		void prerenderTable();

		bool cueBallHit(CueStick *stick, Ball *ball);
		void stickMoved(CueStick *stick, Fixed oldAngle,
						Fixed newAngle);

		void updateAimLines();

	private:
		bool animLoop();

		void createHole(int num, Fixed x, Fixed y);

	private:
		PoolGame *game;

		Hole *holes[6];
		Side *sides[6];

		QList<Ball> inPlayBalls;
		QList<Ball> outOfPlayBalls;
		QList<Sprite> obstacles;
		Ball *cueBall;

		TableTheme theme;
		SDL_Surface *ballShadow;

		int tableR, tableB, tableG;

		Side *topSide,      *bottomSide;
		Side *topLeftSide,  *bottomLeftSide;
		Side *topRightSide, *bottomRightSide;
};

#endif /* _POOLTABLE_H_ */
