#include "PowerMeter.h"

extern ImageLoader *imageLoader;

PowerMeter::PowerMeter(SDL_Surface *screen, PoolGame *game)
	: Sprite(screen, IMAGE_METER_BG, true), game(game), maxLevel(100), level(0)
{
	bar = imageLoader->load(IMAGE_METER_BAR, false);
}

const char *
PowerMeter::getName() const
{
	return "PowerMeter";
}

void
PowerMeter::setLevel(Fixed level)
{
	if (level > maxLevel)
		this->level = maxLevel;
	else
		this->level = level;

	render(true);
}

Fixed
PowerMeter::getLevel() const
{
	return level;
}

Fixed
PowerMeter::getMaxLevel() const
{
	return maxLevel;
}

bool
PowerMeter::mousePressed(unsigned int x, unsigned int y)
{
	return mouseDragged(x, y);
}

bool
PowerMeter::mouseReleased(unsigned int, unsigned int)
{
	return false;
}

bool
PowerMeter::mouseDragged(unsigned int, unsigned int y)
{
	Fixed level;

	level = (getHeight() - (Fixed(y) - getY())) / getHeight() * maxLevel;

	setLevel(level);

	render();

	return true;
}

void
PowerMeter::render(bool update)
{
	SDL_Rect rect, destRect;
	int y, h;

	game->getRoom()->render(getRect(), false);

	Sprite::render(false);

	h = (level / maxLevel * getHeight()).intVal();
	y = getY().intVal() + getHeight().intVal() - h;

	rect.x = 0;
	rect.y = 0;
	rect.w = bar->w;
	rect.h = h;

	destRect.x = getX().intVal() + 5;
	destRect.y = y;
	destRect.w = bar->w;
	destRect.h = h;

	SDL_BlitSurface(bar, &rect, getScreen(), &destRect);

	if (update)
		SDL_UpdateRect(getScreen(), getX().intVal(), getY().intVal(),
					   getWidth().intVal(), getHeight().intVal());
}
