#ifndef _SPRITE_H_
#define _SPRITE_H_

#include <SDL/SDL.h>
#include "Fixed.h"
#include "MouseListener.h"

class MouseListener;
class SDL_Surface;

class Sprite : public MouseListener
{
	public:
		Sprite(SDL_Surface *screen, Uint16 imgNum, bool alphaChannel = false);
		Sprite(SDL_Surface *screen);
		virtual ~Sprite() = 0;

		virtual const char *getName() const = 0;

		void moveTo(Fixed x, Fixed y);
		void moveBy(Fixed x, Fixed y);

		void setX(Fixed x);
		void setY(Fixed y);
		Fixed getX() const;
		Fixed getY() const;

		Fixed getWidth() const;
		Fixed getHeight() const;

		SDL_Rect getRect() const;

		bool containsPoint(unsigned int x, unsigned int y) const;

		void setAnimating(bool animating);
		bool isAnimating() const;

		void setImage(SDL_Surface *image);
		SDL_Surface *getImage() const;
		SDL_Surface *getScreen() const;

		virtual bool collidesWith(Sprite *sprite);

		virtual void render(bool update = true);
		virtual void render(SDL_Rect rect, bool update = true);
		virtual void nextFrame() {}

		SDL_Rect getPrevRect() const { return prevRect; }

		bool mousePressed(unsigned int x, unsigned int y);
		bool mouseReleased(unsigned int x, unsigned int y);
		bool mouseDragged(unsigned int x, unsigned int y);

	protected:
		virtual void moved() {}

	private:
		Fixed x, y;
		bool animating;
		SDL_Rect prevRect;
		SDL_Surface *screen, *image;
};

#include "BilliardZ.h"

#endif /* _SPRITE_H_ */
