#include "Vec2D.h"
#include <math.h>

Vec2D::Vec2D()
{
	setVec(0, 0);
}

Vec2D::Vec2D(const Vec2D &vec)
{
	dx = vec.getDx();
	dy = vec.getDy();
}

Vec2D::Vec2D(Fixed dx, Fixed dy)
{
	setVec(dx, dy);
}

Vec2D::~Vec2D()
{
}

Vec2D
Vec2D::addVec(const Vec2D &vec) const
{
	return Vec2D(getDx() + vec.getDx(), getDy() + vec.getDy());
}

Vec2D
Vec2D::subVec(const Vec2D &vec) const
{
	return Vec2D(getDx() - vec.getDx(), getDy() - vec.getDy());
}

Vec2D
Vec2D::mulVec(const Vec2D &vec) const
{
	return Vec2D(getDx() * vec.getDx(), getDy() * vec.getDy());
}

Vec2D
Vec2D::scaleVec(Fixed scale) const
{
	return Vec2D(getDx() * scale, getDy() * scale);
}

Vec2D
Vec2D::unitVec() const
{
	Fixed d = abs();

	return Vec2D(getDx() / d, getDy() / d);
}

Fixed
Vec2D::abs() const
{
	return (getDx().square() + getDy().square()).sqrt();
}

Fixed
Vec2D::absSquare() const
{
	return getDx().square() + getDy().square();
}

Fixed
Vec2D::dotProd(const Vec2D &vec) const
{
	return (getDx() * vec.getDx() + getDy() * vec.getDy());
}

Vec2D
Vec2D::proj(const Vec2D &vec) const
{
	Fixed f;

	f = vec.getDx().square() + vec.getDy().square();

	if (f > 0)
		return vec.scaleVec(dotProd(vec) / f);

	return *this;
}

Vec2D
Vec2D::setVec(Fixed dx, Fixed dy)
{
	this->dx = dx;
	this->dy = dy;

	return *this;
}

Vec2D
Vec2D::setVec(const Vec2D &vec)
{
	return setVec(vec.getDx(), vec.getDy());
}

Fixed
Vec2D::getDx() const
{
	return dx;
}

Fixed
Vec2D::getDy() const
{
	return dy;
}
